#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdbool.h>
//#include <util/atomic.h>
#include "main.h"
#include "uart.h"

#define TX_BUFFER_MASK (TxBufferSize - 1)

static char TxBuf0[TxBufferSize];		// transmission char buffer
static char RxBuf0[RxBufferSize];		// receive char buffer

extern        char RecordBuf[RecordBufSize];
volatile uint8_t Tx0Wp;	// write pointers
volatile uint8_t Tx0Rp;	// read pointers
volatile uint8_t Rx0Wp;	// write pointers
volatile uint8_t Rx0Rp;	// write pointers


//===========================================================================
// Forward Declarations
void uartputc(char data);

//===========================================================================
void uart_init(void)		// initialize serial interfaces
{

uint8_t	ucsrct,scratch;
// first UART

	ucsrct=0;
	scratch=PARITY;
	switch (scratch) {
		case	'E':	ucsrct |= 0x20;
						break;
		case	'O':	ucsrct |= 0x30;
						break;
	}
	scratch=BITS;	
	switch (scratch) {
		case	  7:	ucsrct |=0x02;
						break;

		case	  8:	ucsrct |=0x06;
						break;
	}
	scratch=STOPB;
	switch (scratch) {
		case	  2:	ucsrct |=0x8;
						break;
	}
	
	UBRRH=(uint8_t)(((F_CPU/(16L*BAUD))-1)>>8);
	UBRRL=(uint8_t)(((F_CPU/(16L*BAUD))-1));
	UCSRA=0;
	UCSRC=ucsrct | (1<<URSEL);
	UCSRB=(1<<RXCIE)|(1<<RXEN)|(1<<TXEN);
	Tx0Wp=Rx0Wp=0;	// initialize write pointers
	Tx0Rp=Rx0Rp=0;	// initialize read pointers

#if FLOW == NONE
	RTS_ON;
#endif

}
//===========================================================================
// USART puts routine
void uartputs(char *s)
{
	while (*s) {
		uartputc(*s++);
	}
}
//===========================================================================
// USART transmission routine
void uartputc(char data)
{
	uint8_t	p;
	p=(Tx0Wp+1)&TX_BUFFER_MASK;
	while(p==Tx0Rp);			// wait for space in the buffer
	TxBuf0[p]=data;
	Tx0Wp=p;
	UCSRB |= (1<<UDRIE);			// UDRE interrupt enable
}
//===========================================================================
// USART Transmission interrupt routine
ISR(USART_UDRE_vect)
{
	uint8_t	p;
	if(Tx0Rp != Tx0Wp)
	{
		p=(Tx0Rp + 1) & TX_BUFFER_MASK;
		Tx0Rp=p;
#if FLOW == RTSCTS
		loop_until_bit_is_clear(PIND,CTS);
									// remember, this kind of flow control is
									// blocking the cpu in the interrupt service
									// routine, so we should think about enabling
									// interrupts here..
#endif
		UDR=TxBuf0[p];				// next char from buffer
	}
	else
		UCSRB&=~(1<<UDRIE);  		// disable UDRE interrupt
}

//===========================================================================
// USART 0 Receive Interrupt routine
ISR(USART_RXC_vect) {
uint8_t c;

	c=UDR;
	if (bit_is_clear(UCSRA, FE))	// Frame Error ?
	{
		RxBuf0[Rx0Wp++] = c;
		if(Rx0Wp >= RxBufferSize)	// rotate
			Rx0Wp=0;
#if FLOW == RTSCTS
		if(intbits.rx0_chars++ >= RxBufferSize-2)
			RTS_OFF;				// pull RTS down
#else
		intbits.rx0_chars++;
#endif

	}
}


//===========================================================================

uint8_t uartgetc() {
uint8_t c;

	while(!intbits.rx0_chars);
	cli();
	c=RxBuf0[Rx0Rp++];
	if(Rx0Rp >= RxBufferSize)
		Rx0Rp=0;					// rotate
	sei();
#if FLOW == RTSCTS
	if (intbits.rx0_chars-- <= RxBufferSize-12)
		RTS_ON;					// we are ready to receive again
#else
	intbits.rx0_chars--;
#endif
	return(c);
}

//===========================================================================

void uart_clearrec()
{
	Rx0Rp=Rx0Wp=0;
	intbits.rx0_chars=0;
}
//===========================================================================
